<?php
/**
 * @package   Switips
 * @author    Outcode https://outcode.ru/
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

//jbdump($params);

class plgVmPaymentSwitips extends JPlugin{
  
  public $states = [    // Статусы VirtueMart
    'P' => 'new',       // PENDING - ОЖИДАНИЕ
    'U' => 'new',       // CONFIRMED_BY_SHOPPER - ПОДТВЕРЖДЕНО ПОКУПАТЕЛЕМ
    'C' => 'break',     // CONFIRMED -  ПРОПУСКАЕМ
    'X' => 'canceled',  // CANCELLED - ОТМЕНЕН
    'R' => 'canceled',  // REFUNDED - ОТМЕНЕН 
    'S' => 'break',     // SHIPPED  -  ПРОПУСКАЕМ
    'F' => 'confirmed', // COMPLETED - ЗАВЕРШЕНО
    'D' => 'canceled',  // CANCELLED - ОТМЕНЕН   
  ];
  
  public function __construct(&$subject, $config){
    parent::__construct($subject, $config);
  }

  public function plgVmCouponUpdateOrderStatus($data, $old_order_status){
    $model = VmModel::getModel('orders');
    $order = $model->getMyOrderDetails($data->virtuemart_order_id);    
    
    $app = &JFactory::getApplication(); 

    if($data->order_status == 'P' || $data->order_status == 'U'){    
      $uid = $app->input->cookie->get('switips_uid', '');
    }else{
      $uid = $this->getUid($data->virtuemart_order_id);
    }     

    $commission = $this->getCommission($order['items']);

    $result = $this->sendToSwitips($uid, $order['details']['BT'], $commission);
    $this->saveUid($uid, $order['details']['BT']->virtuemart_order_id);
  }
  
  public function sendToSwitips($uid, $order, $commission = 0){
    $switips = $this->getParams();

    if($uid == ''){ return; }
    if(intval($switips->sw_status) != 1){ return; }     
    if($this->states[$order->order_status] == 'break'){ return; }

    $args = [      
      'secret_key' => $switips->key_api,
      'merchant_id' => $switips->merchant_id,
      'campaign_id' => 0,
      'category_id' => 0,
      'user_id' => $uid,    
      'transaction_id' => $order->virtuemart_order_id,
      'transaction_amount' => $order->order_total,
      'transaction_amount_currency' => $order->order_total,
      'currency' => $this->getCurrencyCode($order->order_currency),
      'tt_date' => strtotime($order->created_on),
      'stat' => $this->states[$order->order_status], 
      'commission_amount' => $commission,
    ]; 

    JLoader::register('Switips', JPATH_ROOT . '/libraries/switipsapi/switips.php');    
    $SwitipsCls = new Switips($switips->url_api);    
    $send = $SwitipsCls->sendRequestPost($args);

    unset($args['secret_key']);
    unset($args['merchant_id']);
    
    $this->logs($args, 'args');
    $this->logs($send, 'send');

    return $send;
  }  
  
  function getCurrencyCode($currency_id){
    $db = JFactory::getDBO();  
    $query = 'SELECT `currency_code_3` FROM  #__virtuemart_currencies WHERE `virtuemart_currency_id`='.intval($currency_id);
    $db->setQuery($query);
    $result = $db->loadAssocList();

    if(isset($result[0]['currency_code_3'])){
      return $result[0]['currency_code_3'];
    }else{
      return 'RUB';
    }
  }
  
  function getCommission($products){
    $db = JFactory::getDBO();

    $return = 0;

    foreach($products as $product){
      $qty = $product->product_quantity;
      $unit_price = $product->product_final_price; // Цена одной единицы товара
      
      $commission = $this->categoryCommissions($product->virtuemart_product_id, $unit_price);        
      
      $query = 'SELECT `commission` FROM  #__virtuemart_products WHERE `virtuemart_product_id`='.intval($product->virtuemart_product_id);
      $db->setQuery($query);
      $result = $db->loadAssocList(); 


      $tmp = (isset($result[0]['commission'])) ? $result[0]['commission'] : 0;
      $product_commission = $this->checCommission($tmp);      

      if($product_commission > 0){
        
        if(strpos($product_commission, '%') !== false){
          $product_commission = str_replace('%', '', $product_commission);
          $product_commission = ($unit_price/100)*$product_commission;          
        }
        
        $commission = $product_commission;        
      }

      $return += $qty * $commission;
    }

    return $return;
  }
  
  public function categoryCommissions($product_id, $unit_price){
    $db = JFactory::getDBO();

    $query = "SELECT pr_cat.virtuemart_category_id, cat.commission FROM `#__virtuemart_product_categories` AS pr_cat
              LEFT JOIN `#__virtuemart_categories` AS cat ON pr_cat.virtuemart_category_id = cat.virtuemart_category_id
              WHERE pr_cat.virtuemart_product_id=".(int)$product_id;
    $db->setQuery($query);
    $data = $db->loadAssocList();

    $catalog_commission = 0;
    $prev_commission = 0;
    $next_commission = 0;
    
    foreach($data as $d){
      $next_commission = $this->checCommission($d['commission']);
      
      if($next_commission == 0){
        continue;
      }

      if(strpos($next_commission, '%') !== false){
        $next_commission = str_replace('%', '', $next_commission);
        $next_commission = ($unit_price/100)*$next_commission; 
      }      
      
      // max значение
      $catalog_commission = ($next_commission > $prev_commission) ? $next_commission : $prev_commission;
      $prev_commission = $catalog_commission;
    }
    
    return $catalog_commission;    
    
  }
  
  public function checCommission($value = 0){
    $pattern = '/[^0-9 %]+/msiu';
    if(preg_match($pattern, $value)){      
      return 0; // Найдены запрещенные символы
    }else{
      return str_replace(' ', '', $value);
    }  
  }   
  
  public function saveUid($uid, $category_id = 0){
    $db = &JFactory::getDBO();
    $data = new stdClass();
    $data->order_id = $category_id;
    $data->uid = $uid;

    $result = $db->insertObject('#__switips_uid', $data);
    return $result;
  } 
  
  public function getUid($order_id){
    $db = &JFactory::getDBO();
    
    if(!$db->connected()) {
      return null;
    }else{  

      $query = 'SELECT `uid` FROM  #__switips_uid WHERE `order_id`='.intval($order_id);
      $db->setQuery($query);
      $result = $db->loadAssocList();  
      
      if(isset($result[0]['uid'])){
        return $result[0]['uid'];
      }else{
        return '';
      }
      
    } 
  } 
  
  public function getParams(){
    $db = &JFactory::getDBO();
  
    $query = "SELECT `params` FROM  #__extensions WHERE `element`='com_switips'";
    $db->setQuery($query);
    $result = $db->loadAssocList(); 
    
    if(isset($result[0]['params'])){
      return json_decode($result[0]['params']);
    }       
  }

  public function logs($arg, $suf){
    $file = __DIR__.'/'.date('d.m.y', time()).'--'.$suf.'.txt';
    
    if(file_exists($file)){
      $current = file_get_contents($file);
    }else{
      $current = '';
    }

    $current .= '========='.date('d.m.y-h:m:s', time()).'==========='.PHP_EOL;
    $current .= var_export($arg, true).PHP_EOL;
    file_put_contents($file, $current);     
  }

}


